/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, 2008, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Outputter
//
// Douglas Thrift
//
// $Id: Outputter.hpp 372 2008-08-23 11:00:12Z douglas $

#ifndef _Outputter_hpp_
#define _Outputter_hpp_

#include "Search.hpp"
#include "Searcher.hpp"

class Outputter
{
private:
	string headerFile, bodyFile, footerFile, notfoundFile, pagesFile;
	bool query, results;
	double time;
	vector<Ranker> webpages;
	size_t numWebpages, numPages;
	void header(const string& query, size_t page,
		const vector<string>& common, bool and_, bool or_,
		const string& ignore);
	void body();
	void footer(const string& query, size_t page,
		const vector<string>& common, bool and_, bool or_,
		const string& ignore);
	void notfound(const string& query, size_t keywords);
	string pages(string query, size_t page);
	string range(size_t page);
	string total();
	string duration();
	string manycommon(const vector<string>& common);
	void tag(string& line, const char* tag, const string& replacement);
	void conditional(string& line, ifstream& fin, const char* tag, bool condition);
public:
	Outputter(const string& headerFile, const string& bodyFile,
		const string& footerFile, const string& notfoundFile,
		const string& pagesFile) : headerFile(headerFile), bodyFile(bodyFile),
		footerFile(footerFile), notfoundFile(notfoundFile),
		pagesFile(pagesFile) {}
	~Outputter() {}
	void output(Searcher& searcher, size_t page);
};

#endif // _Outputter_hpp_
