/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, 2008, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Ranker
//
// Douglas Thrift
//
// $Id: Ranker.hpp 372 2008-08-23 11:00:12Z douglas $

#ifndef _Ranker_hpp_
#define _Ranker_hpp_

#include "Search.hpp"
#include "Page.hpp"

class Ranker : public Page
{
private:
	typedef map<size_t, size_t> Map;
	enum state { all, title, url, text };
	enum { sampleMax = 160 };
	string sample;
	Map occurrencesTitle;
	Map occurrencesDescription;
	Map occurrencesText;
	size_t value;
	vector<string> required;
	vector<string> excluded;
	vector<string> eitherOr;
	size_t requiredValue;
	size_t excludedValue;
	size_t eitherOrValue;
	string site;
	state allIn;
	string lowerURL;
	string lowerAddress;
	string lowerTitle;
	string lowerText;
	void rank();
	void checkRequired();
	void checkExcluded();
	void checkEitherOr();
	size_t find(string word, const string& where);
	size_t find(string word, const string& where, Map& occurrences);
	size_t phrase(const string& phrase, const string& where);
	size_t phrase(const string& phrase, const string& where,
		Map& occurrences);
	size_t phrase(const vector<string>& words, size_t word,
		size_t& begin, bool start, const string& where);
	size_t phrase(const vector<string>& words, size_t word,
		size_t& begin, bool start, const string& where, Map& occurrences);
	size_t evaluate(vector<size_t>& ins);
	void decrap(string& crap);
public:
	Ranker(Page& page) : Page(page), value(0), requiredValue(0),
		excludedValue(0), eitherOrValue(0), allIn(all) {}
	~Ranker() {}
	void rank(vector<string> query);
	void setSample();
	string getTitle();
	string getDescription();
	string& getSample() { return sample; }
	bool operator==(const size_t number) const;
	bool operator==(const Ranker& ranker) const;
	bool operator!=(const size_t number) const;
	bool operator!=(const Ranker& ranker) const;
	bool operator<(const size_t number) const;
	bool operator<(const Ranker& ranker) const;
	bool operator>(const size_t number) const;
	bool operator>(const Ranker& ranker) const;
	bool operator<=(const size_t number) const
	{
		return *this == number || *this < number;
	}
	bool operator<=(const Ranker& ranker) const
	{
		return *this == ranker || *this < ranker;
	}
	bool operator>=(const size_t number) const
	{
		return *this == number || *this > number;
	}
	bool operator>=(const Ranker& ranker) const
	{
		return *this == ranker || *this > ranker;
	}
// friends:
	friend bool operator==(const size_t number, const Ranker& ranker)
	{
		return ranker == number;
	}
	friend bool operator!=(const size_t number, const Ranker& ranker)
	{
		return ranker != number;
	}
	friend bool operator>(const size_t number, const Ranker& ranker)
	{
		return ranker < number;
	}
	friend bool operator<(const size_t number, const Ranker& ranker)
	{
		return ranker > number;
	}
	friend bool operator>=(const size_t number, const Ranker& ranker)
	{
		return ranker <= number;
	}
	friend bool operator<=(const size_t number, const Ranker& ranker)
	{
		return ranker >= number;
	}
};

#endif // _Ranker_h_
